Option Explicit On 
Option Strict On
Option Compare Binary

Imports System
Imports System.IO
Imports System.Xml
Imports System.Xml.Serialization

' <remarks>
'   Stellt Funktionalitt zum Zugriff auf 
'   bestimmte Einstellungen, die aus einer Datei geladen
'   bzw. in eine Datei gespeichert werden knnen, bereit.
' </remarks>
Public Class Configuration
    Private m_cfgConfiguration As ConfigurationData
    Private m_cfgDefaultConfiguration As ConfigurationData

    ' <summary>
    '   Erstellt eine neue Instanz der Klasse und 
    '   weist den Eigenschaften die in den Parametern
    '   angegebenen Werte zu.
    ' </summary>
    ' <param name="cfgDefaultData">Wert der 
    '    Eigenschaft <c>DefaultData</c>.</param>
    Public Sub New( _
      ByVal cfgDefaultData As ConfigurationData)
        Me.Configuration = cfgDefaultData
        Me.DefaultConfiguration = cfgDefaultData
    End Sub

    ' <summary>
    '   Speichert die Einstellungen in die 
    '   im Parameter <paramref name="strFileName"/>
    '   angegebene Datei.
    ' </summary>
    ' <param name="strFileName">Name der Datei, 
    '  in die die Einstellungen gespeichert werden.</param>
    ' <exception cref="System.Exception">Wird ausgelst, 
    '   wenn beim Speichern ein Fehler auftritt.</exception>
    Public Sub Save(ByVal strFileName As String)
        Dim writer As XmlWriter
        Try
            Dim ser As XmlSerializer = _
              New XmlSerializer(GetType(ConfigurationData))
            writer = New XmlTextWriter(strFileName, _
              New System.Text.UTF8Encoding())
            ser.Serialize(writer, m_cfgConfiguration)
        Catch ex As Exception
            Throw ex
        Finally
            writer.Close()
        End Try
        writer.Close()
    End Sub

    ' <summary>
    '   Ldt die Einstellungen aus der im Parameter 
    '   <paramref name="strFileName"/> angegebenen
    '   Datei.
    ' </summary>
    ' <param name="strFileName">Name der Datei, die 
    '    die Einstellungen enthlt.</param>
    ' <exception cref="System.Exception">
    '   Wird ausgelst, wenn ein Fehler beim 
    '   Laden auftritt und daher die Standard-Einstellungen
    '   verwendet werden.
    ' </exception>
    Public Sub Load(ByVal strFileName As String)
        Try
            Dim ser As XmlSerializer = _
              New XmlSerializer(GetType(ConfigurationData))
            Dim fs As FileStream = _
              New FileStream(strFileName, FileMode.Open)
            Dim reader As XmlReader = _
              New XmlTextReader(fs)
            m_cfgConfiguration = _
              CType(ser.Deserialize(reader), ConfigurationData)
            reader.Close()
        Catch

            ' Es ist ein Fehler beim Lesen aufgetreten 
            ' (Datei existiert nicht, kann nicht gelesen
            ' werden oder besitzt ein ungltiges Format, 
            ' das nicht deserialsiert werden kann).
            ' In diesem Fall werden die im Konstruktor 
            ' der Klasse angegebenen Standardeinstellungen
            ' benutzt.
            m_cfgConfiguration = m_cfgDefaultConfiguration
            Throw New Exception("Error loading configuration, " & _
              "using default configuration data instead.")
        End Try
    End Sub

    ' <summary>
    '   Gibt die Konfigurationseinstellungen an 
    '   oder gibt sie zurck.
    ' </summary>
    ' <value>Objekt, das die 
    '    Konfigurationseinstellungen enthlt.</value>
    Public Property Configuration() As ConfigurationData
        Get
            Return m_cfgConfiguration
        End Get
        Set(ByVal Value As ConfigurationData)
            m_cfgConfiguration = Value
        End Set
    End Property

    ' <summary>
    '   Gibt die Standard-Konfigurationseinstellungen 
    '   an oder gibt sie zurck.
    ' </summary>
    ' <value>Objekt, das die Standard-Konfigurationseinstellungen 
    '   enthlt.</value>
    Public Property DefaultConfiguration() As ConfigurationData
        Get
            Return m_cfgDefaultConfiguration
        End Get
        Set(ByVal Value As ConfigurationData)
            m_cfgDefaultConfiguration = Value
        End Set
    End Property
End Class